/****************************** Transmission ****************************/

uint8_t DDR=0;
uint8_t tx_buff[DATA_SIZE];
uint8_t tx_idx;
#define rx_idx  tx_idx
#define rx_buff tx_buff
#define DATA_INT        SBIT( TIMSK0, OCIE0B )

ISR( TIM0_COMPB_vect, ISR_NOBLOCK )           // Transmit/Sync interrupt
{ 
  uint8_t *dp;					// locals for faster access

  if( DDR ){
    if( tx_idx < DATA_SIZE * 8 ){
      OCR0B += (uint16_t)(BIT_TIME + 0.5);      // 0-bit: 1T
      dp = &tx_buff[tx_idx++ >> 3];              // get byte address
      if( *dp & 0x80 )                          // msb first
        OCR0B += (uint16_t)(BIT_TIME + 0.5);    // 1-bit: 2T
      *dp <<= 1;                                // shift bits
      return;
    }
    OCR0B += (uint16_t)(BIT_TIME * 4 + 0.5);    // Sync:  4T
    TCCR0A = 1<<COM0B1;// | 1<<COM0B0;             // set on compare
///////////////////////////////////////////////////////////////////////////////////////////
	if( DATA_PIN == 1 )
      return;
  }else{
    if( rx_idx != DATA_SIZE_I * 8 ){              // receive successful
      rx_idx = 255;                             // ready for start transition
      return;
    }
    PCMSK  = 0;                                 // disable further receive
  }
  DATA_INT = 0;                                 // disable interrupt
}


void tx_data( uint8_t addr, uint8_t data )
{ DDR = 1;
  PCMSK  = 0;    				// disable receive
  //DATA_DDR = 0;                                 // input
  DATA_DDR = 1;
  DATA = 1;                                     // pullup on
  //while( DATA_PIN == 0 );                       // wait until pin high
  OCR0B = TCNT0 + (uint16_t)(BIT_TIME * 4 + 0.5);      // sync: 4T
  TCCR0A = 1<<COM0B0;                           // toggle on compare
  DATA_DDR = 1;                                 // tx output on
  TIFR0 = 1<<OCF0B;                             // clear earlier interrupts
  DATA_INT = 1;                                 // enable tx interrupt
  volatil(tx_idx) = 0;                          // point to first bit
  volatil(tx_buff[0]) = addr;
  volatil(tx_buff[1]) = data;
  while( DATA_INT );                            // until tx done
}


/****************************** Receiving *******************************/

ISR( PCINT0_vect, ISR_NOBLOCK )                  // Receive interrupt
{ 
  uint8_t *dp;					                 // locals for faster access
  uint8_t i;                                	 // locals for faster access
  uint8_t t;

  t = TCNT0;
  if( rx_idx >= DATA_SIZE_I * 8 ) 			     // on start or stop bit
  {             
    rx_idx = 0;                                  // point to first bit
  }

  else
  {
    dp = &rx_buff[rx_idx++ >> 3];                // get byte address
    i = *dp << 1;                                // shift bits
    if((uint8_t)(OCR0B - t) < (uint8_t)(BIT_TIME * 1.4 + 0.5))  // if pulse > 1.4T
      i++;                                       // msb first
    *dp = i;
  }

  OCR0B = t + (uint8_t)(BIT_TIME * 2.8 + 0.5);   // detect 4T
}

#define	rx_busy() PCMSK                          // 0 = data received

void rx_init( void )
{ IN_DDR = 0;
  IN = 1;
  
  DDR = 0;
  //DATA_DDR = 0;         //!!!!!!!!!                         // input
  //DATA = 0;              //!!!!!!!                        // pullup on

  OCR0B = TCNT0 + (uint16_t)(BIT_TIME * 2.8 - 0.5);    // detect 4T
  volatil(rx_idx) = 255;                         // detect sync transition
  TIFR0 = 1<<OCF0B;                              // clear earlier interrupts
  DATA_INT = 1;                                  // enable sync interrupt
  PCMSK = 1<<PCINT0;
  GIFR  = 1<<PCIF;                               // clear earlier interrupts
  GIMSK = 1<<PCIE;                               // pin change interrupt
}

/****************************** PWM *************************************/

ISR( TIM0_COMPA_vect )
{
   if( OCR0A != 0xFF )
     PWM = 1;								    // LED on
}


ISR( TIM0_OVF_vect )
{
  if( OCR0A != 0x00 )
    PWM = 0;                                    // LED off
}

void pwm_init( void )
{
  OCR0B  = 0xFF;                                // 0%
  TIMSK0 = 1<<OCIE0A | 1<<TOIE0;
  TCCR0A = 1<<COM0B1;// | 1<<COM0B0;               // set data pin on compare
  TCCR0B = PRESCALER_SET;                       // start T2
}
